//---------------------------------------------------------------------------
#include "SubstrateDepSeedSurvival.h"
#include "SimManager.h"
#include "TreePopulation.h"
#include "ParsingFunctions.h"
#include "GridBase.h"
#include "Plot.h"
#include <stdio.h>
//---------------------------------------------------------------------------
///////////////////////////////////////////////////////////////////////////////
// Constructor
// Edit history:
// -------------
// April 18, 2005 - Created (LEM)
/////////////////////////////////////////////////////////////////////////////
clSubstrateDepSeedSurvival::clSubstrateDepSeedSurvival( clSimManager * p_oSimManager ) : clBehaviorBase( p_oSimManager ),
     clWorkerBase( p_oSimManager )
     {
       try
       {
         strcpy( m_cNameString, "establishment" );

         mp_iIndexes = NULL;
         mp_fCanScarifiedSoilFav = NULL;
         mp_fCanTipUpFav = NULL;
         mp_fCanFreshLogFav = NULL;
         mp_fCanDecayedLogFav = NULL;
         mp_fCanForestFloorFav = NULL;
         mp_fGapScarifiedSoilFav = NULL;
         mp_fGapTipUpFav = NULL;
         mp_fGapFreshLogFav = NULL;
         mp_fGapDecayedLogFav = NULL;
         mp_fGapForestFloorFav = NULL;
         mp_iSubstrFavCode = NULL;
         mp_iSeedGridCode = NULL;

         mp_oSubstrateFavGrid = NULL;

         //Allowed file types
         m_iNumAllowedTypes = 2;
         mp_iAllowedFileTypes = new int[m_iNumAllowedTypes];
         mp_iAllowedFileTypes[0] = parfile;
         mp_iAllowedFileTypes[1] = detailed_output;

         //Versions
         m_fVersionNumber = 1;
         m_fMinimumVersionNumber = 1;
       }
       catch ( modelErr & err )
       {
         throw( err );
       }
       catch ( modelMsg & msg )
       {
         throw( msg );
       } //non-fatal error
       catch ( ... )
       {
         modelErr stcErr;
         stcErr.iErrorCode = UNKNOWN;
         strcpy( stcErr.cFunction, "clSubstrateDepSeedSurvival::clSubstrateDepSeedSurvival" );
         throw( stcErr );
       }
}

///////////////////////////////////////////////////////////////////////////////
// Destructor
// Edit history:
// -------------
// April 18, 2005 - Created (LEM)
/////////////////////////////////////////////////////////////////////////////
clSubstrateDepSeedSurvival::~clSubstrateDepSeedSurvival()
{

  delete[] mp_fCanScarifiedSoilFav;
  delete[] mp_fCanTipUpFav;
  delete[] mp_fCanFreshLogFav;
  delete[] mp_fCanDecayedLogFav;
  delete[] mp_fCanForestFloorFav;
  delete[] mp_fGapScarifiedSoilFav;
  delete[] mp_fGapTipUpFav;
  delete[] mp_fGapFreshLogFav;
  delete[] mp_fGapDecayedLogFav;
  delete[] mp_fGapForestFloorFav;
  delete[] mp_iIndexes;
  delete[] mp_iSubstrFavCode;
  delete[] mp_iSeedGridCode;
}

///////////////////////////////////////////////////////////////////////////////
// SetNameData
// Edit history:
// -------------
// April 18, 2005 - Created (LEM)
///////////////////////////////////////////////////////////////////////////////
void clSubstrateDepSeedSurvival::SetNameData( char * cNameString )
{
  try
  {
    if ( strcmp( cNameString, "No Gap Substrate Seed Survival" ) == 0 )
    {
      m_bUsesGap = false;
    }
    else if ( strcmp( cNameString, "Gap Substrate Seed Survival" ) == 0 )
    {
      m_bUsesGap = true;
    }
    else
    { //error!
      modelErr stcErr;
      stcErr.iErrorCode = BAD_DATA;
      strcpy( stcErr.cFunction, "clSubstrateDepSeedSurvival::SetNameData" );
      strcpy( stcErr.cMoreInfo, strcat( "Unrecognized name ", cNameString ) );
      throw( stcErr );
    }
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clSubstrateDepSeedSurvival::SetNameData" );
    throw( stcErr );
  }
}

///////////////////////////////////////////////////////////////////////////////
// GetParameterFileData
// Edit history:
// -------------
// April 18, 2005 - Created (LEM)
/////////////////////////////////////////////////////////////////////////////
void clSubstrateDepSeedSurvival::GetParameterFileData( xercesc::DOMDocument * p_oDoc )
{
  DOMElement * p_oElement = p_oDoc->getDocumentElement();
  clTreePopulation * p_oPop = ( clTreePopulation * ) mp_oSimManager->GetPopulationObject( "treepopulation" );
  floatVal * p_fTemp = NULL; //for getting species-specific values
  short int iNumSpecies = p_oPop->GetNumberOfSpecies(), i;

  //Declare our data arrays
  mp_iIndexes = new short int[iNumSpecies];
  //Load the indexes array
  //Make the list of indexes
  for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
    mp_iIndexes[mp_iWhatSpecies[i]] = i;

  //Declare the canopy array values
  mp_fCanScarifiedSoilFav = new float[m_iNumBehaviorSpecies];
  mp_fCanTipUpFav = new float[m_iNumBehaviorSpecies];
  mp_fCanFreshLogFav = new float[m_iNumBehaviorSpecies];
  mp_fCanDecayedLogFav = new float[m_iNumBehaviorSpecies];
  mp_fCanForestFloorFav = new float[m_iNumBehaviorSpecies];

  //Set up our temp array - pre-load with this behavior's species
  p_fTemp = new floatVal[m_iNumBehaviorSpecies];
  for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
    p_fTemp[i].code = mp_iWhatSpecies[i];

  //Canopy scarified soil favorability
  FillSpeciesSpecificValue( p_oElement, "es_scarifiedSoilCanopyFav", "es_sscfVal", p_fTemp,
       m_iNumBehaviorSpecies, p_oPop, true );
  //Now transfer the values to the permanent array
  for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
    mp_fCanScarifiedSoilFav[mp_iIndexes[p_fTemp[i].code]] = p_fTemp[i].val;

  //Canopy tip-up mound favorability
  FillSpeciesSpecificValue( p_oElement, "es_tipUpCanopyFav", "es_tucfVal", p_fTemp, m_iNumBehaviorSpecies, p_oPop, true );
  //Now transfer the values to the permanent array
  for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
    mp_fCanTipUpFav[mp_iIndexes[p_fTemp[i].code]] = p_fTemp[i].val;

  //Canopy fresh log favorability
  FillSpeciesSpecificValue( p_oElement, "es_freshLogCanopyFav", "es_flcfVal", p_fTemp, m_iNumBehaviorSpecies, p_oPop, true );
  //Now transfer the values to the permanent array
  for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
    mp_fCanFreshLogFav[mp_iIndexes[p_fTemp[i].code]] = p_fTemp[i].val;

  //Canopy decayed log favorability
  FillSpeciesSpecificValue( p_oElement, "es_decayedLogCanopyFav", "es_dlcfVal", p_fTemp, m_iNumBehaviorSpecies, p_oPop, true );
  //Now transfer the values to the permanent array
  for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
    mp_fCanDecayedLogFav[mp_iIndexes[p_fTemp[i].code]] = p_fTemp[i].val;

  //Canopy forest floor favorability
  FillSpeciesSpecificValue( p_oElement, "es_forestFloorCanopyFav", "es_ffcfVal", p_fTemp,
       m_iNumBehaviorSpecies, p_oPop, true );
  //Now transfer the values to the permanent array
  for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
    mp_fCanForestFloorFav[mp_iIndexes[p_fTemp[i].code]] = p_fTemp[i].val;

  //Verify that all values are between 0 and 1
  for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
  {
    if ( mp_fCanScarifiedSoilFav[i] < 0 || mp_fCanScarifiedSoilFav[i] > 1 || mp_fCanTipUpFav[i] < 0 || mp_fCanTipUpFav[i] > 1
         || mp_fCanFreshLogFav[i] < 0 || mp_fCanFreshLogFav[i] > 1 || mp_fCanDecayedLogFav[i] < 0
         || mp_fCanDecayedLogFav[i] > 1 || mp_fCanForestFloorFav[i] < 0 || mp_fCanForestFloorFav[i] > 1 )
         {
           modelErr stcErr;
           strcpy( stcErr.cFunction, "clSubstrateDepSeedSurvival::GetParameterFileData" );
           strcpy( stcErr.cMoreInfo, "All substrate favorabilities must be between 0 and 1." );
           stcErr.iErrorCode = BAD_DATA;
           throw( stcErr );
    }
  }


  if ( m_bUsesGap )
  {
    //Declare the gap array values
    mp_fGapScarifiedSoilFav = new float[m_iNumBehaviorSpecies];
    mp_fGapTipUpFav = new float[m_iNumBehaviorSpecies];
    mp_fGapFreshLogFav = new float[m_iNumBehaviorSpecies];
    mp_fGapDecayedLogFav = new float[m_iNumBehaviorSpecies];
    mp_fGapForestFloorFav = new float[m_iNumBehaviorSpecies];

    //Load the gap values
    //Gap scarified soil favorability
    FillSpeciesSpecificValue( p_oElement, "es_scarifiedSoilGapFav", "es_ssgfVal", p_fTemp,
         m_iNumBehaviorSpecies, p_oPop, true );
    //Now transfer the values to the permanent array
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fGapScarifiedSoilFav[mp_iIndexes[p_fTemp[i].code]] = p_fTemp[i].val;

    //Gap tip-up mound favorability
    FillSpeciesSpecificValue( p_oElement, "es_tipUpGapFav", "es_tugfVal", p_fTemp, m_iNumBehaviorSpecies, p_oPop, true );
    //Now transfer the values to the permanent array
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fGapTipUpFav[mp_iIndexes[p_fTemp[i].code]] = p_fTemp[i].val;

    //Gap fresh log favorability
    FillSpeciesSpecificValue( p_oElement, "es_freshLogGapFav", "es_flgfVal", p_fTemp, m_iNumBehaviorSpecies, p_oPop, true );
    //Now transfer the values to the permanent array
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fGapFreshLogFav[mp_iIndexes[p_fTemp[i].code]] = p_fTemp[i].val;

    //Gap decayed log favorability
    FillSpeciesSpecificValue( p_oElement, "es_decayedLogGapFav", "es_dlgfVal", p_fTemp, m_iNumBehaviorSpecies, p_oPop, true );
    //Now transfer the values to the permanent array
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fGapDecayedLogFav[mp_iIndexes[p_fTemp[i].code]] = p_fTemp[i].val;

    //Gap forest floor favorability
    FillSpeciesSpecificValue( p_oElement, "es_forestFloorGapFav", "es_ffgfVal", p_fTemp,
         m_iNumBehaviorSpecies, p_oPop, true );
    //Now transfer the values to the permanent array
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fGapForestFloorFav[mp_iIndexes[p_fTemp[i].code]] = p_fTemp[i].val;

    //Verify that all values are between 0 and 1
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
    {
      if ( mp_fGapScarifiedSoilFav[i] < 0 || mp_fGapScarifiedSoilFav[i] > 1 || mp_fGapTipUpFav[i] < 0
           || mp_fGapTipUpFav[i] > 1 || mp_fGapFreshLogFav[i] < 0 || mp_fGapFreshLogFav[i] > 1 || mp_fGapDecayedLogFav[i] < 0
           || mp_fGapDecayedLogFav[i] > 1 || mp_fGapForestFloorFav[i] < 0 || mp_fGapForestFloorFav[i] > 1 )
           {
             modelErr stcErr;
             strcpy( stcErr.cFunction, "clSubstrateDepSeedSurvival::GetParameterFileData" );
             strcpy( stcErr.cMoreInfo, "All substrate favorabilities must be between 0 and 1." );
             stcErr.iErrorCode = BAD_DATA;
             throw( stcErr );
      }
    }
  }

  delete[] p_fTemp;
}

///////////////////////////////////////////////////////////////////////////////
// GetData
// Edit history:
// -------------
// April 18, 2005 - Created (LEM)
/////////////////////////////////////////////////////////////////////////////
void clSubstrateDepSeedSurvival::GetData( xercesc::DOMDocument * p_oDoc )
{
  try
  {

    GetParameterFileData( p_oDoc );

    //Set up the grids
    SetupGrids();
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clSubstrateDepSeedSurvival::GetData" );
    throw( stcErr );
  }
}

///////////////////////////////////////////////////////////////////////////////
// Action
// Edit history:
// -------------
// April 18, 2005 - Created (LEM)
///////////////////////////////////////////////////////////////////////////////
void clSubstrateDepSeedSurvival::Action()
{
  clPlot *p_oPlot = mp_oSimManager->GetPlotObject();
  float fCover, //value of a particular substrate cover
       fIndex, //favorability index
       fSeedX, fSeedY, //coordinates of seed
       fXOrig, fYOrig, //coordinates of origin of a grid cell
       fNumSeeds, //number of seeds per grid
       fSurvivingSeeds, //number of seeds surviving per grid
       fXCellLength = mp_oSeedGrid->GetLengthXCells(), fYCellLength = mp_oSeedGrid->GetLengthYCells(),
       fScarifiedSoilFav, fForestFloorFav, fTipUpFav, fFreshLogFav, fDecayedLogFav;
  short int iNumXCells = mp_oSubstrateFavGrid->GetNumberXCells(), iNumYCells = mp_oSubstrateFavGrid->GetNumberYCells(),iX, iY, iSp, i, j, k; //loop counters
  bool bIsGap; //whether or not a grid cell is under gap cover

  for ( iX = 0; iX < iNumXCells; iX++ )
    for ( iY = 0; iY < iNumYCells; iY++ )
    {

      //Get the gap status for this grid cell if gap is being used
      if ( m_bUsesGap )
      {
        mp_oSeedGrid->GetValueOfCell( iX, iY, m_iIsGapCode, & bIsGap );
      }
      else
      {
        bIsGap = false;
      }
      //Calculate the index for each species
      for ( k = 0; k < m_iNumBehaviorSpecies; k++ )
      {
        if ( bIsGap )
        {
          fScarifiedSoilFav = mp_fGapScarifiedSoilFav[mp_iIndexes[mp_iWhatSpecies[k]]];
          fForestFloorFav = mp_fGapForestFloorFav[mp_iIndexes[mp_iWhatSpecies[k]]];
          fTipUpFav = mp_fGapTipUpFav[mp_iIndexes[mp_iWhatSpecies[k]]];
          fFreshLogFav = mp_fGapFreshLogFav[mp_iIndexes[mp_iWhatSpecies[k]]];
          fDecayedLogFav = mp_fGapDecayedLogFav[mp_iIndexes[mp_iWhatSpecies[k]]];
        }
        else
        {
          fScarifiedSoilFav = mp_fCanScarifiedSoilFav[mp_iIndexes[mp_iWhatSpecies[k]]];
          fForestFloorFav = mp_fCanForestFloorFav[mp_iIndexes[mp_iWhatSpecies[k]]];
          fTipUpFav = mp_fCanTipUpFav[mp_iIndexes[mp_iWhatSpecies[k]]];
          fFreshLogFav = mp_fCanFreshLogFav[mp_iIndexes[mp_iWhatSpecies[k]]];
          fDecayedLogFav = mp_fCanDecayedLogFav[mp_iIndexes[mp_iWhatSpecies[k]]];
        }

        //Get scarified soil
        mp_oSubstrateGrid->GetValueOfCell( iX, iY, m_iScarifiedSoilCode, & fCover );
        //Multiply it by scarified soil favorability for our cover
        fIndex = fCover * fScarifiedSoilFav;

        //Get forest floor, multiply it by the favorability, and add it in
        mp_oSubstrateGrid->GetValueOfCell( iX, iY, m_iForestFloorCode, & fCover );
        fIndex += fCover * fForestFloorFav;

        //Get tip-up, multiply it by the favorability, and add it in
        mp_oSubstrateGrid->GetValueOfCell( iX, iY, m_iTipUpCode, & fCover );
        fIndex += fCover * fTipUpFav;

        //Get fresh log, multiply it by the favorability, and add it in
        mp_oSubstrateGrid->GetValueOfCell( iX, iY, m_iFreshLogCode, & fCover );
        fIndex += fCover * fFreshLogFav;

        //Get decayed log, multiply it by the favorability, and add it in
        mp_oSubstrateGrid->GetValueOfCell( iX, iY, m_iDecayedLogCode, & fCover );
        fIndex += fCover * fDecayedLogFav;

        //Put the index in our grid
        mp_oSubstrateFavGrid->SetValueOfCell( iX, iY, mp_iSubstrFavCode[mp_iWhatSpecies[k]], fIndex );
      }
    } //end of for (j = 0; j < iNumYCells; j++)

  //Calculate seed survival
  iNumXCells = mp_oSeedGrid->GetNumberXCells();
  iNumYCells = mp_oSeedGrid->GetNumberYCells();

  //Loop through each grid cell
  for ( iX = 0; iX < iNumXCells; iX++ )
  {
    for ( iY = 0; iY < iNumYCells; iY++ )
    {

      //Get the origin of this cell (min X and Y values)
      fXOrig = fXCellLength * iX;
      fYOrig = fYCellLength * iY;

      //Loop through each behavior species
      for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      {

        iSp = mp_iWhatSpecies[i];
        fSurvivingSeeds = 0;

        //Get the number of seeds in this cell for this species
        mp_oSeedGrid->GetValueOfCell( iX, iY, mp_iSeedGridCode[iSp], & fNumSeeds );

        //Disperse the seeds produced
        for ( j = 0; j < fNumSeeds; j++ )
        {

          //Get a random X and Y value for this seed within the grid cell
          fSeedX = p_oPlot->CorrectX( fXOrig + ( clModelMath::GetRand() * fXCellLength ) );
          fSeedY = p_oPlot->CorrectY( fYOrig + ( clModelMath::GetRand() * fYCellLength ) );

          //Compute substrate germination probability - compare
          //substrate favorability index to a random number
          mp_oSubstrateFavGrid->GetValueAtPoint( fSeedX, fSeedY, mp_iSubstrFavCode[iSp], & fIndex );

          if ( clModelMath::GetRand() < fIndex )
          {
            //seed survives
            fSurvivingSeeds++;
          }

          //Set the surviving seeds back into the grid cell
          mp_oSeedGrid->SetValueOfCell( iX, iY, mp_iSeedGridCode[iSp], fSurvivingSeeds );
        }
      }
    }
  }
}

///////////////////////////////////////////////////////////////////////////////
// SetUpGrids()
// Edit history:
// -------------
// April 18, 2005 - Created (LEM)
/////////////////////////////////////////////////////////////////////////////
void clSubstrateDepSeedSurvival::SetupGrids()
{
  try
  {

    clPopulationBase * p_oTemp = mp_oSimManager->GetPopulationObject( "treepopulation" );
    clTreePopulation * p_oPop = ( clTreePopulation * ) p_oTemp;
    char cLabel[MAX_GRID_LABEL_SIZE];
    short int iLengthXCells, iLengthYCells, //length of cells for grid
         iNumXCells, iNumYCells, //number of cells for grid
         iNumSpecies = p_oPop->GetNumberOfSpecies(), i, j; //loop counters


    //////////////////////////////////////////////
    // First the substrate grid
    //////////////////////////////////////////////
      mp_oSubstrateGrid = mp_oSimManager->GetGridObject( "Substrate" );
      if ( !mp_oSubstrateGrid )
      {
        modelErr stcErr;
        stcErr.iErrorCode = CANT_FIND_OBJECT;
        strcpy( stcErr.cFunction, "clSubstrateDepSeedSurvival::SetupGrids" );
        strcpy( stcErr.cMoreInfo, "Substrate behavior required." );
        throw( stcErr );
      }

      //Get the data member codes for the substrate grid
      m_iScarifiedSoilCode = mp_oSubstrateGrid->GetFloatDataCode( "gapmoss" );
      m_iForestFloorCode = mp_oSubstrateGrid->GetFloatDataCode( "canmoss" );
      m_iTipUpCode = mp_oSubstrateGrid->GetFloatDataCode( "organic" );
      m_iFreshLogCode = mp_oSubstrateGrid->GetFloatDataCode( "freshlog" );
      m_iDecayedLogCode = mp_oSubstrateGrid->GetFloatDataCode( "mossylog" );

      //If any of the codes are -1 throw an error
      if ( -1 == m_iScarifiedSoilCode || -1 == m_iForestFloorCode || -1 == m_iTipUpCode || -1 == m_iFreshLogCode
           || -1 == m_iDecayedLogCode )
           {
             modelErr stcErr;
             stcErr.iErrorCode = BAD_DATA;
             strcpy( stcErr.cFunction, "clSubstrateDepSeedSurvival::SetUpGrids" );
             strcpy( stcErr.cMoreInfo, "Unexpected grid data return code." );
             throw( stcErr );
      }

      if ( NULL == mp_oSubstrateFavGrid )
      {
        //Get the length of the X cells and Y cells of substrate grid
        iLengthXCells = mp_oSubstrateGrid->GetLengthXCells();
        iLengthYCells = mp_oSubstrateGrid->GetLengthYCells();

        //Create the substrate favorability grid
        mp_oSubstrateFavGrid = mp_oSimManager->CreateGrid( "Substrate Favorability", //grid name
             0, //number of int data members
             iNumSpecies, //number of float data members
             0, //number of char data members
             0, //number of bool data members
             iLengthXCells, //X cell length
             iLengthYCells ); //Y cell length

        mp_iSubstrFavCode = new short int[iNumSpecies];

        //Register the data members
        for ( i = 0; i < iNumSpecies; i++ )
        {
          sprintf( cLabel, "%s%d", "Favorability Index", i );
          mp_iSubstrFavCode[i] = mp_oSubstrateFavGrid->RegisterFloat( cLabel );
        }
      }

    //////////////////////////////////////////////
    // Now the dispersed seeds grid
    //////////////////////////////////////////////
    mp_oSeedGrid = mp_oSimManager->GetGridObject( "Dispersed Seeds" );
    if ( NULL == mp_oSeedGrid )
    {
      modelErr stcErr;
      stcErr.iErrorCode = CANT_FIND_OBJECT;
      strcpy( stcErr.cFunction, "clSubstrateDepSeedSurvival::SetupGrids" );
      strcpy( stcErr.cMoreInfo, "Disperse behaviors are required if establishment is to be used." );
      throw( stcErr );
    }
    mp_iSeedGridCode = new short int[iNumSpecies];
    for ( i = 0; i < iNumSpecies; i++ )
    {
      sprintf( cLabel, "%s%d", "seeds_", i );
      mp_iSeedGridCode[i] = mp_oSeedGrid->GetFloatDataCode( cLabel );
      if ( -1 == mp_iSeedGridCode[i] )
      {
        modelErr stcErr;
        stcErr.iErrorCode = BAD_DATA;
        strcpy( stcErr.cFunction, "clSubstrateDepSeedSurvival::SetupGrids" );
        strcpy( stcErr.cMoreInfo, "Unexpected grid data return code." );
        throw( stcErr );
      }
    }

    if ( m_bUsesGap )
    {
      m_iIsGapCode = mp_oSeedGrid->GetBoolDataCode( "Is Gap" );
      if ( -1 == m_iIsGapCode )
      {
        modelErr stcErr;
        stcErr.iErrorCode = BAD_DATA;
        strcpy( stcErr.cFunction, "clSubstrateDepSeedSurvival::SetupGrids" );
        strcpy( stcErr.cMoreInfo, "Unexpected grid data return code." );
        throw( stcErr );
      }
    }
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clSubstrateDepSeedSurvival::SetupGrids" );
    throw( stcErr );
  }
}

