//---------------------------------------------------------------------------
#include "LightWriter.h"
#include "SimManager.h"
#include "TreePopulation.h"
#include <fstream.h>
//---------------------------------------------------------------------------

/*/////////////////////////////////////////////////////////////////////////////
Constructor()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

/////////////////////////////////////////////////////////////////////////////*/
clLightWriter::clLightWriter(clSimManager *p_oSimManager) :
     clBehaviorBase(p_oSimManager), clWorkerBase(p_oSimManager) {

  //Allowed file types
  m_iNumAllowedTypes = 2;
  mp_iAllowedFileTypes = new int[m_iNumAllowedTypes];
  mp_iAllowedFileTypes[0] = parfile;
  mp_iAllowedFileTypes[1] = detailed_output;

  //Versions
  m_fVersionNumber = 1;
  m_fMinimumVersionNumber = 1;

  strcpy(m_cNameString, "lightwriter");
}
//---------------------------------------------------------------------------

/*/////////////////////////////////////////////////////////////////////////////
Action()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

/////////////////////////////////////////////////////////////////////////////*/
void clLightWriter::Action() {
  clTreePopulation *p_oPop;
  clPopulationBase *p_oTemp = mp_oSimManager->GetPopulationObject("treepopulation");
  clTreeSearch *p_oAllTrees;
  clTree *p_oTree;
  fstream outfile("Light.xls", ios::out | ios::app);
  float fX, fY, fDiam, fLight, fHeight;
  int iTimestep = mp_oSimManager->GetCurrentTimestep();
  int iYlr, iYls;
  short int iCode, iSp, iTp;

  //If this is the first timestep write out the header
  if (iTimestep == 1)
    outfile << "\nX\tY\tSpecies\tDiam\tHeight\tLight\n";

  outfile << "\nTimestep " << iTimestep << "\n";

  p_oPop = (clTreePopulation*) p_oTemp;

  //Get all the trees
  p_oAllTrees = p_oPop->Find("all");
  p_oTree = p_oAllTrees->NextTree();
  while (p_oTree) {

    iSp = p_oTree->GetSpecies();
    iTp = p_oTree->GetType();



    if (iTp == clTreePopulation::sapling) {

    //X & Y
    p_oTree->GetValue(p_oPop->GetXCode(iSp, iTp), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(iSp, iTp), &fY);
    p_oTree->GetValue(p_oPop->GetHeightCode(iSp, iTp), &fHeight);
    if (iTp == clTreePopulation::seedling)
      p_oTree->GetValue(p_oPop->GetDiam10Code(iSp, iTp), &fDiam);
    else
      p_oTree->GetValue(p_oPop->GetDbhCode(iSp, iTp), &fDiam);

    p_oTree->GetValue(p_oPop->GetFloatDataCode("Light", iSp, iTp), &fLight);

    outfile << fX << "\t" << fY << "\t" << iSp << "\t" << fDiam << "\t"
            << fHeight << "\t" << fLight << "\n";


    }


    p_oTree = p_oAllTrees->NextTree();
  }

/*  //Do the seedlings and saplings and write out ylr and yls
  p_oAllTrees = p_oPop->Find("type=1,2");
  p_oTree = p_oAllTrees->NextTree();
  while (p_oTree) {
    iSp = p_oTree->GetSpecies();
    iTp = p_oTree->GetType();
    //X & Y
    p_oTree->GetValue(p_oPop->GetXCode(iSp, iTp), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(iSp, iTp), &fY);
    //Ylr and Yls
    p_oTree->GetValue(p_oPop->GetIntDataCode("ylr", iSp, iTp), &iYlr);
    p_oTree->GetValue(p_oPop->GetIntDataCode("yls", iSp, iTp), &iYls);

    outfile << fX << "\t" << fY << "\t" << iYlr << "\t" << iYls << "\n";

    p_oTree = p_oAllTrees->NextTree();
  } */

  outfile.close();
}
//---------------------------------------------------------------------------
